@@ -49,9 +49,7 @@ objects, as well as the proper SQL representation of basic Python
 types (None==NULL).
 
 There are a number of DB-specific SQL features that this does not
-implement.  There are a bunch of normal ANSI features also not present
--- particularly left joins, among others.  You may wish to only use
-this to generate ``WHERE`` clauses.
+implement.  There are a bunch of normal ANSI features also not present.
 
 See the bottom of this module for some examples, and run it (i.e.
 ``python sql.py``) to see the results of those examples.
@@ -62,18 +60,28 @@ See the bottom of this module for some examples, and run it (i.e.
 ## Constants
 ########################################
 
+import fnmatch
+import operator
+import re
+import threading
+import types
+import weakref
+
+import classregistry
+from converters import sqlrepr, registerConverter
+
+
 class VersionError(Exception):
     pass
 class NoDefault:
     pass
 
-import re, fnmatch
-import operator
-import threading
-import types
-import classregistry
 
-from converters import sqlrepr, registerConverter
+class SQLObjectState(object):
+    def __init__(self, soObject, connection=None):
+        self.soObject = weakref.proxy(soObject)
+        self.connection = connection
+
 
 safeSQLRE = re.compile(r'^[a-zA-Z_][a-zA-Z0-9_\.]*$')
 def sqlIdentifier(obj):
@@ -321,44 +329,38 @@ class Field(SQLExpression):
         return executor.field(self.tableName, self.fieldName)
 
 class SQLObjectField(Field):
-    def __init__(self, tableName, fieldName, original):
-        self.original = original
+    def __init__(self, tableName, fieldName, original, soClass, column):
         Field.__init__(self, tableName, fieldName)
-
-registerConverter(SQLObjectField, SQLExprConverter)
-
-
-class UnicodeField(SQLObjectField):
-    def __init__(self, tableName, fieldName, original, column):
-        SQLObjectField.__init__(self, tableName, fieldName, original)
+        self.original = original
+        self.soClass = soClass
         self.column = column
+    def _from_python(self, value):
+        column = self.column
+        if not isinstance(value, SQLExpression) and column and column.from_python:
+            value = column.from_python(value, SQLObjectState(self.soClass))
+        return value
     def __eq__(self, other):
         if other is None:
             return ISNULL(self)
-        if isinstance(other, unicode):
-            other = other.encode(self.column.dbEncoding)
+        other = self._from_python(other)
         return SQLOp('=', self, other)
     def __ne__(self, other):
         if other is None:
             return ISNOTNULL(self)
-        if isinstance(other, unicode):
-            other = other.encode(self.column.dbEncoding)
+        other = self._from_python(other)
         return SQLOp('<>', self, other)
     def startswith(self, s):
-        if isinstance(s, unicode):
-            s = s.encode(self.column.dbEncoding)
+        s = self._from_python(s)
         return STARTSWITH(self, s)
     def endswith(self, s):
-        if isinstance(s, unicode):
-            s = s.encode(self.column.dbEncoding)
+        s = self._from_python(s)
         return ENDSWITH(self, s)
     def contains(self, s):
-        if isinstance(s, unicode):
-            s = s.encode(self.column.dbEncoding)
+        s = self._from_python(s)
         return CONTAINSSTRING(self, s)
 
+registerConverter(SQLObjectField, SQLExprConverter)
 
-registerConverter(UnicodeField, SQLExprConverter)
 
 class Table(SQLExpression):
     FieldClass = Field
@@ -378,7 +380,6 @@ class Table(SQLExpression):
 
 class SQLObjectTable(Table):
     FieldClass = SQLObjectField
-    UnicodeFieldClass = UnicodeField
 
     def __init__(self, soClass):
         self.soClass = soClass
@@ -394,10 +395,7 @@ class SQLObjectTable(Table):
             return self._getattrFromID(attr)
         elif attr in self.soClass.sqlmeta.columns:
             column = self.soClass.sqlmeta.columns[attr]
-            if hasattr(column, "dbEncoding"):
-                return self._getattrFromUnicodeColumn(column, attr)
-            else:
-                return self._getattrFromColumn(column, attr)
+            return self._getattrFromColumn(column, attr)
         elif attr+'ID' in [k for (k, v) in self.soClass.sqlmeta.columns.items() if v.foreignKey]:
             attr += 'ID'
             column = self.soClass.sqlmeta.columns[attr]
@@ -406,18 +404,15 @@ class SQLObjectTable(Table):
             raise AttributeError("%s instance has no attribute '%s'" % (self.soClass.__name__, attr))
 
     def _getattrFromID(self, attr):
-        return self.FieldClass(self.tableName, self.soClass.sqlmeta.idName, attr)
+        return self.FieldClass(self.tableName, self.soClass.sqlmeta.idName, attr, self.soClass, None)
 
     def _getattrFromColumn(self, column, attr):
-        return self.FieldClass(self.tableName, column.dbName, attr)
-
-    def _getattrFromUnicodeColumn(self, column, attr):
-        return self.UnicodeFieldClass(self.tableName, column.dbName, attr, column)
+        return self.FieldClass(self.tableName, column.dbName, attr, self.soClass, column)
 
 class SQLObjectTableWithJoins(SQLObjectTable):
 
     def __getattr__(self, attr):
-        if attr+'ID' in [k for (k,v) in self.soClass.sqlmeta.columns.items() if v.foreignKey]:
+        if attr+'ID' in [k for (k, v) in self.soClass.sqlmeta.columns.items() if v.foreignKey]:
             column = self.soClass.sqlmeta.columns[attr+'ID']
             return self._getattrFromForeignKey(column, attr)
         elif attr in [x.joinMethodName for x in self.soClass.sqlmeta.joins]:
@@ -482,7 +477,7 @@ class AliasTable(Table):
     def __init__(self, table, alias=None):
         if hasattr(table, "sqlmeta"):
             tableName = SQLConstant(table.sqlmeta.table)
-        elif isinstance(table, (Select,Union)):
+        elif isinstance(table, (Select, Union)):
             assert alias is not None, "Alias name cannot be constructed from Select instances, please provide 'alias' kw."
             tableName = Subquery('', table)
             table = None
@@ -532,7 +527,7 @@ class Union(SQLExpression):
                 if isinstance(t, Table):
                     t = t.tableName
                 if not isinstance(t, SQLExpression):
-                    t = SQLConstant(t.sqlmeta.table)
+                    t = SQLConstant(t)
             tabs.append(t)
         self.tables = tabs
 
@@ -841,6 +836,11 @@ def _IN(item, list):
     return SQLOp("IN", item, list)
 
 def IN(item, list):
+    from sresults import SelectResults # Import here to avoid circular import
+    if isinstance(list, SelectResults):
+        query = list.queryForSelect()
+        query.ops['items'] = [list.sourceClass.q.id]
+        list = query
     if isinstance(list, Select):
         return INSubquery(item, list)
     else:
@@ -916,7 +916,7 @@ class _LikeQuoted:
            raise TypeError, "expected str, unicode or SQLExpression, got %s" % type(s)
 
 def _quote_like_special(s, db):
-    if db == 'postgres':
+    if db in ('postgres', 'rdbhost'):
         escape = r'\\'
     else:
         escape = '\\'
@@ -1182,6 +1182,7 @@ class RLIKE(LIKE):
         'maxdb': 'RLIKE',
         'mysql': 'RLIKE',
         'postgres': '~',
+        'rdbhost': '~',
         'sqlite': 'REGEXP'
     }
 
@@ -1250,7 +1251,7 @@ class ImportProxy(SQLExpression):
         self.sqlmeta = _Delay_proxy(table=_DelayClass(self, clsName))
         self.q = self
         self.soClass = None
-        classregistry.registry(registry).addClassCallback(clsName,lambda foreign, me: setattr(me, 'soClass', foreign), self)
+        classregistry.registry(registry).addClassCallback(clsName, lambda foreign, me: setattr(me, 'soClass', foreign), self)
 
     def __nonzero__(self):
         return True
